/************************************************************************
 *
 *  StyleWithPropertiesConverterHelper.java
 *
 *  Copyright: 2002-2014 by Henrik Just
 *
 *  This file is part of Writer2LaTeX.
 *  
 *  Writer2LaTeX is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Writer2LaTeX is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Writer2LaTeX.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *  Version 1.6 (2014-10-24)
 *
 */

package w2phtml.xhtml.style;

import java.util.Enumeration;
//import java.util.Hashtable;

import w2phtml.office.OfficeReader;
import w2phtml.office.StyleWithProperties;
import w2phtml.xhtml.Converter;
import w2phtml.xhtml.StyleInfo;
import w2phtml.xhtml.XhtmlConfig;
import w2phtml.xhtml.XhtmlStyleMapItem;
import w2phtml.xhtml.style.properties.Properties;

/**
 * <p>This is an abstract class to convert an OpenDocument style family
 * represented by <code>StyleWithProperties</code> to CSS2 styles.</p>
 */
public abstract class StyleWithPropertiesParser extends StyleParser {

    boolean inlineCSS = false;
		/** Create a new <code>StyleWithPropertiesConverterHelper</code>
     *  @param ofr an <code>OfficeReader</code> to read style information from
     *  @param config the configuration to use
     *  @param converter the main <code>Converter</code> class
     *  @param nType the type of xhtml to use
     */
    public StyleWithPropertiesParser(OfficeReader ofr, XhtmlConfig config,
        Converter converter, int nType) {
        super(ofr,config,converter,nType);
        inlineCSS = config.inlineCSS();
    }

    /** Apply a style, either by converting the style or by applying the
     *  style map from the configuarion
     *  @param sStyleName name of the OpenDocument style
     *  @param info the <code>StyleInfo</code> object to add information to
     */
  	public void readStyle(String sStyleName, StyleInfo info) {
  		StyleWithProperties style = (StyleWithProperties) getStyles().getStyle(sStyleName);
  		if (style == null) {

  			return;
  		}
  		info.sTagName = getStyleTag();
  		if (config.multilingual()) {
  			applyLang(style, info);
  		}
  		applyDirection(style, info);
  		if (style.isAutomatic()) {
  			// Apply parent style + hard formatting
  			readStyle(style.getParentName(), info);
  			applyProperties(style, info.props, false);
  		} else {

  			String displayName = style.getDisplayName();
  			if (styleMap.contains(displayName)) {
  				// Apply attributes as specified in style map from user
  				XhtmlStyleMapItem map = styleMap.get(displayName);
  				if (map.sElement.length() > 0) {
  					info.sTagName = map.sElement;
  				}
  				if (!"(none)".equals(map.sCss)) {
  					info.sClass = map.sCss;
  				}
  			} else {
  				// Generate class name from display name
  				styleNames.addName(displayName);
  				info.sClass = getClassNamePrefix() + styleNames.getName(displayName);
    			applyProperties(style, info.props, false);
  				
  			}
  		}
  	}
  	
  	public void readParentStyle(String styleName, StyleInfo info) {
  		if (!inlineCSS || styleName == null) {
  			return;
  		}
  		StyleWithProperties style = (StyleWithProperties) getStyles().getStyle(styleName);
  		if (info.sClass != null  && !info.sClass.equals(styleName)) {
  			StyleWithProperties classStyle = (StyleWithProperties) getStyles().getStyle(info.sClass);
    		if (classStyle != null) {
    			applyProperties(classStyle, info.props, true);
    			String parentName = classStyle.getParentName();
    			if (parentName != null && !parentName.equals(styleName)) {
    				if (info.unknownAncestor(parentName)) {
    					info.addAncestor(parentName);
    					readParentStyle(parentName, info);	
    				}
    			}
    		}
  		}
  		if (style == null) {
  			return;
  		}
  		String displayName = style.getDisplayName();
  		String exportName = styleNames.getName(displayName);
  		if (exportName != null && !exportName.trim().equals("null")) {
    		applyProperties(style, info.props, true);
  		}
  		String parentName = style.getParentName();
			if (parentName != null && !parentName.equals(styleName)) {
				if (info.unknownAncestor(parentName)) {
					info.addAncestor(parentName);
					readParentStyle(parentName, info);
				}
			}
  	}
	
    /** Convert style information for used styles
     *  @param sIndent a String of spaces to add before each line
     */
  	public String composeStyleDeclarations() {
  		if (inlineCSS) {
  			return "";
  		}
  		StringBuilder buf = new StringBuilder();
  		Enumeration<String> names = styleNames.keys();
  		while (names.hasMoreElements()) {
  			String sDisplayName = names.nextElement();
  			buf.append(composeDeclaration(sDisplayName));
  		}
  		return buf.toString();
  	}

		public String composeDeclaration( String displayName) {
  		String declaration = "";
			StyleWithProperties style = (StyleWithProperties) getStyles().getStyleByDisplayName(displayName);
			String selector = composeSelector(displayName);
			String declarationBlock = composeDeclarationBlock(displayName);
			if (style!= null && !style.isAutomatic() && !selector.isEmpty() && !declarationBlock.isEmpty()) {
				declaration = createDeclaration(selector, declarationBlock);
				// TODO: Create a method "getStyleDeclarationsInner"
				// to be used by eg. FrameStyleConverter
			}
			return declaration.toString();
		}

		public String composeDeclarationBlock(String displayName) {
			StyleWithProperties style = (StyleWithProperties) getStyles().getStyleByDisplayName(displayName);
			if (style == null) {
				return "";
			}
			Properties props = new Properties(";");
			
			applyProperties(style, props, true);
			return props.toString();
		}

		private String composeSelector(String displayName) {
			styleNames.addName(displayName);
			return getStyleTag() + "." + getClassNamePrefix() + styleNames.getName(displayName);
		}
	
    /** Return a prefix to be used in generated css class names
     *  @return the prefix
     */
    public String getClassNamePrefix() { return ""; }

    /** Create default tag name to represent a specific style, e.g.
     *  <code>span</code> (text style) or <code>ul</code> (unordered list)
     *  @param style to use
     *  @return the tag name. If the style is null, a default result should be
     *  returned.
     */
    public abstract String getStyleTag();
	
    /** Convert formatting properties for a specific style.
     *  @param style the style to convert
     *  @param props the <code>CSVList</code> object to add information to
     *  @param bInherit true if properties should be inherited from parent style(s)
     */
    public abstract void applyProperties(StyleWithProperties style, Properties props, boolean bInherit);
	

}
